// Compile using 
//     csc FileUtil.cs

using System;
using System.Net;
using System.IO;
using System.Text;


class FileClient {


	static void Main(string[] args)
	{

		FileClientInfo fci;
		FileClient fc;

		if ((args.Length > 2) && (args.Length < 6))
		{
			fci = new FileClientInfo();
			fc = new FileClient();
			
			fci.RemotePath = args[1];
			fci.LocalPath  = args[2];

			// If credentials were given,
			// we should try and use them
			if (args.Length == 5)
			{
				fci.UserName   = args[3];
				fci.Password   = args[4];
			}
			
			// They want to upload a file
			if (args[0].ToLower() == "-u")
			{
				fc.uploadFile(fci);
			}
			// The want to download a file
			else if (args[0].ToLower() == "-d")
			{
				fc.downloadFile(fci);
			}
			else
			{
				ShowUsage();
			}
		
		}
		else
			ShowUsage();


	}

	private static void ShowUsage() {
		
		Console.WriteLine("\r\nExample Usage:");
		Console.WriteLine("FileUtil -[u|d] [RemoteURL] [LocalPath] [UserName] [Password]\r\n");
		Console.WriteLine("** -u = Upload, -d = Download");
		Console.WriteLine("** The UserName and Password values are optional.\r\n");
	}

	private void uploadFile (FileClientInfo info) {

		WebClient client = new WebClient();
		ASCIIEncoding ascii = new ASCIIEncoding();

		byte[] retBytes = null;

		if ((info.UserName != null) || (info.Password != null))
			client.Credentials = new NetworkCredential(info.UserName, info.Password);
	
		try
		{
			// Are we uploading the contents of a directory?
			Console.WriteLine(info.LocalPath);
		
			DirectoryInfo di = new DirectoryInfo(info.LocalPath);

			if((di.Attributes & FileAttributes.Directory)==FileAttributes.Directory)
			{
				foreach (FileInfo file in di.GetFiles())
				{					
					Console.WriteLine ("SENDING: {0}", file.FullName);
					retBytes = client.UploadFile (info.RemotePath, "POST", file.FullName);
					Console.WriteLine ("RESPONSE: {0}", ascii.GetString(retBytes));
				}

			}
			else
			{
				if (!File.Exists(info.LocalPath)) {
					Console.WriteLine(@"File\Path not found!");
					return;
				}
				else
				{
					Console.WriteLine ("SENDING: {0}", info.LocalPath);
					retBytes = client.UploadFile(info.RemotePath, "POST", info.LocalPath);
					Console.WriteLine ("RESPONSE: {0}", ascii.GetString(retBytes));
				}
			}
		}
		catch (Exception eo)
		{
			Console.WriteLine(eo.Message);
		}

		
	}

	private void downloadFile (FileClientInfo info) {

		WebClient client = new WebClient();

		if ((info.UserName != null) || (info.Password != null))
			client.Credentials = new NetworkCredential(info.UserName, info.Password);

		Console.WriteLine ("REQUESTING: {0}", info.RemotePath);
		try
		{
			client.DownloadFile(info.RemotePath, info.LocalPath);
			Console.WriteLine ("RESPONSE SAVED TO: {0}", info.LocalPath);
		}
		catch (Exception eo)
		{
			Console.WriteLine (eo.Message);
		}
	}

	internal struct FileClientInfo {

		FileClientInfo (string RemotePath, string LocalPath, string UserName, string Password) {
			
			_RemotePath = RemotePath;
			_LocalPath  = LocalPath;
			_UserName   = UserName;
			_Password   = Password;
		}
		FileClientInfo (string RemotePath, string LocalPath) {
			
			_RemotePath = RemotePath;
			_LocalPath  = LocalPath;
			_UserName   = null;
			_Password   = null;
		}

		private string _RemotePath;
		private string _LocalPath;
		private string _UserName;
		private string _Password;

		internal string RemotePath {
			get {return _RemotePath;}
			set {_RemotePath = value;}
		}

		internal string LocalPath {
			get {return _LocalPath;}
			set {_LocalPath = value;}
		}

		internal string UserName {
			get {return _UserName;}
			set {_UserName = value;}
		}

		internal string Password {
			get {return _Password;}
			set {_Password = value;}
		}
	}
}
