<%@ WebHandler Language="C#" Class="ImageHandler" %>

using System;
using System.IO;
using System.Web;
using System.Drawing;
using System.Drawing.Imaging;


class ImageHandler : IHttpHandler {
	
	public void ProcessRequest (HttpContext context) {

		CreateImage(ref context);
	}

	public bool IsReusable {

		// Okay to reuse this object
		get {return true;}
	}

	public void CreateImage (ref HttpContext context) {

		HttpResponse Response = context.Response;
		HttpRequest  Request  = context.Request;

		const string FONT_NAME	= "Courier New";
		const int FONT_SIZE	= 12;	
		
		Bitmap bmp = null;
		Graphics g = null;

		// Set some default text to render
		string RenderText = "[TEXT WAS UNDEFINED]";

		// Find out what the caller wants to render
		if ((Request.QueryString.Get("Text") != null) && 
                  (Request.QueryString.Get("Text") != ""))
				RenderText = Request.QueryString.Get("Text");
		
		// Find out what background color is desired
		string BackgroundColorName = Request.QueryString.Get("BGColor");

		Color BackgroundColor = Color.CornflowerBlue;
		try
		{
			if (BackgroundColorName != "")
				BackgroundColor = ColorTranslator.FromHtml
                                  (BackgroundColorName);
		}
		catch {}

		try
		{
			Font font = new Font(FONT_NAME, FONT_SIZE);
			// calculate size of the string.
			bmp = new Bitmap(1,1,PixelFormat.Format32bppPArgb);
			g   = Graphics.FromImage(bmp);
			SizeF sSize = g.MeasureString(RenderText, font);

			// Get the Height and Width of our string
			int nHeight = (int)sSize.Height;
			int nWidth  = (int)sSize.Width;
			bmp = new Bitmap(nWidth, nHeight, 
				PixelFormat.Format32bppPArgb);

			g = Graphics.FromImage(bmp);
			g.FillRectangle(new SolidBrush(BackgroundColor), 
				new Rectangle(0, 0, nWidth, nHeight));

			g.DrawString(RenderText, font, 
				new SolidBrush(Color.White), 0, 0);

			// Create a MemoryStream that will contain the image
			MemoryStream memStream = new MemoryStream();

			// Save the Image to the MemoryStream
			bmp.Save(memStream, ImageFormat.Png);

			// Clear the content from the buffer stream
			Response.Clear();

			// Inform the browser that this is a PNG file
			Response.ContentType	= "IMAGE/PNG";

			// Take the image from the MemoryStream 
			// and write it to the Response Stream
			Response.BinaryWrite(memStream.ToArray());

			// Send buffered output and fire the 
			// EndRequest event of the Application
			Response.End();

		}
		catch {}

		finally 
		{
			// Clean up the GDI+ surface
			if (null != g) 
				g.Dispose();
			
			// Clean up the Bitmap
			if (null != bmp) 
				bmp.Dispose();
		}	
	
	}

}
