' Compile using 
'     vbc FileUtil.cs
Imports System
Imports System.Net
Imports System.IO
Imports System.Text



Class FileClient
   
   'Entry point which delegates to C-style main Private Function
   Public Overloads Shared Sub Main()
      Main(System.Environment.GetCommandLineArgs())
   End Sub
   
   
   
   Overloads Shared Sub Main(args() As String)
      
      Dim fci As FileClientInfo
      Dim fc As FileClient
      
      If args.Length > 2 And args.Length < 6 Then
         fci = New FileClientInfo()
         fc = New FileClient()
         
         fci.RemotePath = args(1)
         fci.LocalPath = args(2)
         
         ' If credentials were given,
         ' we should try and use them
         If args.Length = 5 Then
            fci.UserName = args(3)
            fci.Password = args(4)
         End If
         
         ' They want to upload a file
         If args(0).ToLower() = "-u" Then
            fc.uploadFile(fci)
         ' The want to download a file
         Else
            If args(0).ToLower() = "-d" Then
               fc.downloadFile(fci)
            Else
               ShowUsage()
            End If
         End If 
      Else
         ShowUsage()
      End If 
   End Sub 'Main
    
   
   Private Shared Sub ShowUsage()
      
      Console.WriteLine("Example Usage:")
      Console.WriteLine("FileUtil -[u|d] [RemoteURL] [LocalPath] [UserName] [Password]")
      Console.WriteLine("** -u = Upload, -d = Download")
      Console.WriteLine("** The UserName and Password values are optional.\n")
   End Sub
   
   
   Private Sub uploadFile(info As FileClientInfo)
      
      Dim client As New WebClient()
      Dim ascii As New ASCIIEncoding()
      
      Dim retBytes As Byte() = Nothing
      
      If Not (info.UserName Is Nothing) Or Not (info.Password Is Nothing) Then
         client.Credentials = New NetworkCredential(info.UserName, info.Password)
      End If 
      Try
         ' Are we uploading the contents of a directory?
         Console.WriteLine(info.LocalPath)
         
         Dim di As New DirectoryInfo(info.LocalPath)
         
         If(di.Attributes And FileAttributes.Directory) = FileAttributes.Directory Then
            Dim file As FileInfo
            For Each file In  di.GetFiles()
               Console.WriteLine("SENDING: {0}", file.FullName)
               retBytes = client.UploadFile(info.RemotePath, "POST", file.FullName)
               Console.WriteLine("RESPONSE: {0}", ascii.GetString(retBytes))
            Next file
         
         Else
            If Not File.Exists(info.LocalPath) Then
               Console.WriteLine("File\Path not found!")
               Dim [not] As Path
               Return
            Else
               Console.WriteLine("SENDING: {0}", info.LocalPath)
               retBytes = client.UploadFile(info.RemotePath, "POST", info.LocalPath)
               Console.WriteLine("RESPONSE: {0}", ascii.GetString(retBytes))
            End If
         End If
      Catch eo As Exception
         Console.WriteLine(eo.Message)
      End Try
   End Sub 
       
   Private Sub downloadFile(info As FileClientInfo)
      
      Dim client As New WebClient()
      
      If Not (info.UserName Is Nothing) Or Not (info.Password Is Nothing) Then
         client.Credentials = New NetworkCredential(info.UserName, info.Password)
      End If 
      Console.WriteLine("REQUESTING: {0}", info.RemotePath)
      Try
         client.DownloadFile(info.RemotePath, info.LocalPath)
         Console.WriteLine("RESPONSE SAVED TO: {0}", info.LocalPath)
      Catch eo As Exception
         Console.WriteLine(eo.Message)
      End Try
   End Sub
    
   
   Friend Structure FileClientInfo
    
      
      Sub New(RemotePath As String, LocalPath As String, UserName As String, Password As String)
         
         _RemotePath = RemotePath
         _LocalPath = LocalPath
         _UserName = UserName
         _Password = Password
      End Sub 'New
      
      Sub New(RemotePath As String, LocalPath As String)
         
         _RemotePath = RemotePath
         _LocalPath = LocalPath
         _UserName = Nothing
         _Password = Nothing
      End Sub 'New
      
      Private _RemotePath As String
      Private _LocalPath As String
      Private _UserName As String
      Private _Password As String
      
      
      Friend Property RemotePath() As String
         Get
            Return _RemotePath
         End Get
         Set
            _RemotePath = value
         End Set
      End Property 
      
      Friend Property LocalPath() As String
         Get
            Return _LocalPath
         End Get
         Set
            _LocalPath = value
         End Set
      End Property 
      
      Friend Property UserName() As String
         Get
            Return _UserName
         End Get
         Set
            _UserName = value
         End Set
      End Property 
      
      Friend Property Password() As String
         Get
            Return _Password
         End Get
         Set
            _Password = value
         End Set
      End Property
   End Structure 
End Class