<%@ Page aspcompat=true %>
<%@ Import Namespace="System.IO" %>
<%@ Import Namespace="System.Runtime.InteropServices" %>
<script language="C#" runat="server">
  // Initializes the AVIFile library.
  [DllImport("avifil32.dll", PreserveSig=false)]
  static extern void AVIFileInit();
  
  // Exits the AVIFile library.
  [DllImport("avifil32.dll", PreserveSig=false)]
  static extern void AVIFileExit();
  
  // Opens the specified AVI file and returns a file interface (IAVIFile).
  
  [DllImport("avifil32.dll", PreserveSig=false)]
  static extern void AVIFileOpen(ref IAVIFile ppfile, string szFile,
    uint mode, Guid pclsidHandler);
 
  // Returns a stream interface (IAVIStream) associated with an AVI file.
  [DllImport("avifil32.dll", PreserveSig=false)]
  static extern void AVIFileGetStream(IAVIFile pfile, 
    ref IAVIStream ppavi, int fccType, int lParam);
 
  // Returns the length of the input stream.
  [DllImport("avifil32.dll")]
  static extern int AVIStreamLength(IAVIStream pavi);
  
  // Converts a stream measurement in samples to milliseconds.
  [DllImport("avifil32.dll")]
  static extern int AVIStreamSampleToTime(IAVIStream pavi, int lSample);
  
  //
  // The IAVIFile COM Interface used in the methods above
  //
  [ComImport, InterfaceType(ComInterfaceType.InterfaceIsIUnknown),
    Guid("00020020-0000-0000-c000-000000000046")]
  interface IAVIFile
  {
    void Info(ref AVIFILEINFO pfi, int lSize);
    void GetStream(ref IAVIStream ppStream, int fccType, int lParam);
    void CreateStream(ref IAVIStream ppStream, AVISTREAMINFO psi);
    void WriteData(int ckid, IntPtr lpData, int cbData);
    void ReadData(int ckid, IntPtr lpData, ref int lpcbData);
    void EndRecord();
    void DeleteStream(int fccType, int lParam);
  };
  
  //
  // The IAVIStream COM Interface used in the methods above
  //
  [ComImport, InterfaceType(ComInterfaceType.InterfaceIsIUnknown),
    Guid("00020021-0000-0000-c000-000000000046")]
  interface IAVIStream
  {
    void Create(int lParam1, int lParam2);
    void Info(ref AVISTREAMINFO psi, int lSize);
    [PreserveSig] int FindSample(int lPos, int lFlags);
    void ReadFormat(int lPos, IntPtr lpFormat, ref int lpcbFormat);
    void SetFormat(int lPos, IntPtr lpFormat, int cbFormat);
    void Read(int lStart, int lSamples, IntPtr lpBuffer, int cbBuffer);
    void Write(int lStart, int lSamples, IntPtr lpBuffer, int cbBuffer,
      int dwFlags, ref int plSampWritten, ref int plBytesWritten);
    void Delete(int lStart, int lSamples);
    void ReadData(int fcc, IntPtr lp, ref int lpcb);
    void WriteData(int fcc, IntPtr lp, int cb);
    void SetInfo(int fcc, IntPtr lp, int cb);
  };
  
  //
  // Structures used in the interfaces above.
  //
 
  [StructLayout(LayoutKind.Sequential, CharSet=CharSet.Auto)]
  public struct AVISTREAMINFO
  {
    public int fccType;
    public int fccHandler;
    public int dwFlags;
    public int dwCaps;
    public short wPriority;
    public short wLanguage;
    public int dwScale;
    public int dwRate;
    public int dwStart;
    public int dwLength;
    public int dwInitialFrames;
    public int dwSuggestedBufferSize;
    public int dwQuality;
    public int dwSampleSize;
    public RECT rcFrame;
    public int dwEditCount;
    public int dwFormatChangeCount;
    [MarshalAs(UnmanagedType.ByValTStr, SizeConst=64)]
    public string szName;
  }
 
  [StructLayout(LayoutKind.Sequential, CharSet=CharSet.Auto)]
  public struct AVIFILEINFO
  {
    public int dwMaxBytesPerSec;
    public int dwFlags;
    public int dwCaps;
    public int dwStreams;
    public int dwSuggestedBufferSize;
    public int dwWidth;
    public int dwHeight;
    public int dwScale;
    public int dwRate;
    public int dwLength;
    public int dwEditCount;
    [MarshalAs(UnmanagedType.ByValTStr, SizeConst=64)]
    public string szFileType;
  }
 
    public struct RECT
    {
    public int left;
      public int top;
      public int right;
    public int bottom;
  }
 
  // Constant used when opening the file
  public const uint OF_SHARE_DENY_WRITE = 0x00000020;
 
  void Page_Load(Object sender, EventArgs e)
    {
    if (Page.IsPostBack && fupUpload.PostedFile != null)
      {
      // Save the file if it has a filename and exists...
        if (fupUpload.PostedFile.FileName.Trim().Length > 0 &&
        fupUpload.PostedFile.ContentLength > 0)
      {
        const string strBaseDir = "C:\\My Projects\\Uploaded Files\\";
        string strFileName =
          Path.GetFileName(fupUpload.PostedFile.FileName);
        fupUpload.PostedFile.SaveAs(strBaseDir + strFileName);
 
          IAVIFile pfile = null;
 
        try
        {
            // Initialize the AVIFile library
            AVIFileInit();
        }
          catch (Exception ex)
        {
            lblResults.Text = 
            "<hr>Server Error: Cannot initialize the AVIFile library." +
            "<br>Error Details:<br>" + ex.ToString();
          return;
        }
 
        try
        {
          // Open the AVI file
          AVIFileOpen(ref pfile, strBaseDir + strFileName,
            OF_SHARE_DENY_WRITE, Guid.Empty);
          }
        catch (Exception ex)
        {
            lblResults.Text =
            "<hr>The uploaded file is not a valid AVI or WAV file." +
            "<br>Error Details:<br>" + ex.ToString();
          return;
        }
 
        // Display summary information
 
          AVIFILEINFO fileinfo = new AVIFILEINFO();
 
        try
        {
            // Call a method on the IAVIFile COM interface
            pfile.Info(ref fileinfo, Marshal.SizeOf(fileinfo));
        }
          catch (Exception ex)
        {
            lblResults.Text = "<hr>Unable to get file information." +
            "<br>Error Details:<br>" + ex.ToString();
          return;
        }
 
        double samplesPerSecond = fileinfo.dwRate / fileinfo.dwScale;
          lblResults.Text = "<hr>Information for file <b>" +
          fupUpload.PostedFile.FileName + "</b>:<hr>" +
          "Type: <b>" + fileinfo.szFileType + "</b><br>" +
          "Width: <b>" + fileinfo.dwWidth + "</b><br>" +
          "Height: <b>" + fileinfo.dwHeight + "</b><br>" +
          "Samples/Sec: <b>" + samplesPerSecond + "</b><br>" +
          "# of Streams: <b>" + fileinfo.dwStreams + "</b><br>";
 
        // Look at each stream and print out summary information
        for (int i = 0; i < fileinfo.dwStreams; i++)
        {
          IAVIStream pstream = null;
 
          try
          {
            AVIFileGetStream(pfile, ref pstream, 0, i);
          }
          catch (Exception ex)
          {
            lblResults.Text = "<hr>Cannot get stream #" +
                (i+1).ToString() +
                ".<br>Error Details:<br>" + ex.ToString();
              return;
            }
 
          AVISTREAMINFO streaminfo = new AVISTREAMINFO();

          try
          {
            // Call a method on the IAVIStream COM interface
            pstream.Info(ref streaminfo, Marshal.SizeOf(streaminfo));
          }
          catch (Exception ex)
          {
            lblResults.Text = "<hr>Cannot get information for stream #" +
              (i+1).ToString() + 
              ".<br>Error Details:<br>" + ex.ToString();
              return;
            }
 
          int time = AVIStreamSampleToTime(pstream, 
            AVIStreamLength(pstream));
          int seconds = time / 1000;
          int fraction = time % 1000;
 
          samplesPerSecond = streaminfo.dwRate / streaminfo.dwScale;
 
          lblResults.Text += "<hr><b>Stream #" +
            (i+1).ToString() + "</b><br>" +
            "Name: <b>" + streaminfo.szName + "</b><br>" +
            "Frame Count: <b>" + AVIStreamLength(pstream).ToString() +
              "</b><br>Play Length: <b>" + seconds.ToString() + "." +
            fraction.ToString() + " sec" + "</b><br>" +
            "Samples/Sec: <b>" + samplesPerSecond + "</b><br>";
        }
 
        // Called instead of AVIFileRelease, 
          // since it releases the COM interface.
        Marshal.ReleaseComObject(pfile);
 
        // Release the AVIFile library
        AVIFileExit();
      }
      else
      {
        lblResults.Text = "<hr>You must enter a valid filename!";
      }
    }
  }
</script>
  <html><title>Multimedia File Upload</title>
   <body>
     <form runat="server" EncType="multipart/form-data">
     <b>Select the AVI or WAV file to upload:</b><br>
     <input runat="server" id="fupUpload" type="file" >
     <p>
       <asp:button id="btnSubmit" runat="server" Text="Upload File" />
       <p><asp:label runat="server" id="lblResults" />
   </form>
 </body>
</html>
